#--
#
#################################################################################
# Copyright 2011 CENATIC y IEPALA
#
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas por la Comisión Europea– 
# versiones posteriores de la EUPL (la «Licencia»);
# Solo podrá usarse esta obra si se respeta la Licencia.
# Puede obtenerse una copia de la Licencia en:
#
# http://www.osor.eu/eupl/european-union-public-licence-eupl-v.1.1
#
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye «TAL CUAL»,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones que establece la Licencia.
#################################################################################
#
#++
# Controlador encargado de la gestion de Gastos. Este controlador es utilizado desde las secciones:
# * Sección proyectos: se utiliza para la gestión de los gastos relacionados con proyecto.
#
class GastoProyectosController < ApplicationController

  require 'RMagick'

  # --
  # METODOS DE ENTRADA A PRESUPUESTO. Seleccion de ETAPA
  # ++
  # en proyectos: se redirecciona por defecto a listado_estapas
  def index
    redirect_to :action => :filtrado_ordenado_iniciales
  end

  def filtrado_ordenado_iniciales
    #TODO: esto deberia hacerse con un filtro y no con un horroroso IF
    if @proyecto.etapa.empty? 
      flash[:error] = _("Tiene que definir por lo menos una etapa para poder acceder a la gestion de Gasto.")
      redirect_to :controller => :datos_proyecto, :action => :etapas
    else
      session[:gasto_proyectos_asc_desc] = "ASC"
      session[:gasto_proyectos_cadena_orden] = "fecha"
      session[:gasto_proyectos_filtro_etapa] = "todas"
      session[:gasto_proyectos_filtro_moneda] = "todas"
      session[:gasto_proyectos_filtro_proyecto] = "todos"
      session[:gasto_proyectos_filtro_agente] = "todos"
      session[:gasto_proyectos_filtro_partida] = "todas"
      redirect_to :action => :listado
    end
  end

  # en proyectos: establece los parametros de ordenación
  def ordenado
    session[:gasto_proyectos_asc_desc] = params[:asc_desc] ? params[:asc_desc] : "ASC"
    session[:gasto_proyectos_cadena_orden] = session[:gasto_proyectos_orden] = params[:orden] ? params[:orden] : "fecha"
    session[:gasto_proyectos_cadena_orden] = "partida.codigo" if session[:gasto_proyectos_orden].include?("partida.codigo_nombre")
    session[:gasto_proyectos_cadena_orden] = "gasto_x_proyecto.importe" if session[:gasto_proyectos_orden].include?("importe")
    session[:gasto_proyectos_cadena_orden] = "proyecto_origen_id" if session[:gasto_proyectos_orden].include?("proyecto")
    redirect_to :action => params[:listado] || "listado"
  end

  def filtrado
    session[:gasto_proyectos_filtro_etapa] = params[:filtro][:etapa]
    session[:gasto_proyectos_filtro_moneda] = params[:filtro][:moneda]
    session[:gasto_proyectos_filtro_proyecto] = params[:filtro][:proyecto] if params[:listado] == "cofinanciables"
    session[:gasto_proyectos_filtro_agente] = params[:filtro][:agente]
    session[:gasto_proyectos_filtro_partida] = params[:filtro][:partida]
    redirect_to :action => params[:listado]
  end


  def elementos_filtrado
    origen = caller[0][/.*`([^']*)/,1]
    filtro_etapa = [["Todas", "todas"]] + @proyecto.etapa.sort{ |a, b| a.fecha_inicio <=> b.fecha_inicio }.collect{ |e| [e.nombre, e.id] }
    filtro_agente = [["Todos", "todos"]]  + @proyecto.implementador.collect{ |e| [e.nombre, e.id] }
    filtro_moneda = [["Todas", "todas"]]  + @proyecto.moneda.collect{ |e| [e.nombre, e.id] }
    filtro_proyecto = [["Todos", "todos"]] + @proyecto.proyecto_cofinanciador.collect{ |p| [p.nombre, p.id] }
    filtro_partida = [["Todas", "todas"]] + Partida.find(:all).collect {|p| [p.codigo_nombre(@proyecto.id), p.id]}
    @opciones_filtrado = [	{:rotulo =>  _("Selecciona etapa: "), :nombre => "etapa", :opciones => filtro_etapa},
				{:rotulo =>  _("Selecciona moneda: "), :nombre => "moneda", :opciones => filtro_moneda},
				{:rotulo =>  _("Selecciona agente: "), :nombre => "agente", :opciones => filtro_agente},
				{:rotulo =>  _("Seleccione partida: "), :nombre => "partida", :opciones => filtro_partida} ]
    @opciones_filtrado.push({:rotulo =>  _("Seleccione proyecto: "), :nombre => "proyecto", :opciones => filtro_proyecto}) if origen=="cofinanciables"

    @estado_filtrado = [ (session[:gasto_proyectos_filtro_etapa] == "todas" ? "Cualquier etapa" : @etapa.nombre + " (" + @etapa.fecha_inicio.to_s + "/" + @etapa.fecha_fin.to_s + ")"),   
                       (session[:gasto_proyectos_filtro_moneda] == "todas" ? "Cualquier moneda" : Moneda.find(session[:gasto_proyectos_filtro_moneda]).nombre), 
                       (session[:gasto_proyectos_filtro_agente] == "todos" ? "Cualquier agente" : Agente.find(session[:gasto_proyectos_filtro_agente]).nombre),
		       (session[:gasto_proyectos_filtro_partida] == "todas" ? "Cualquier partida" : Partida.find(session[:gasto_proyectos_filtro_partida]).nombre) ] 
    @estado_filtrado.push(session[:gasto_proyectos_filtro_proyecto] == "todos" ? "Cualquier proyecto" : Proyecto.find(session[:gasto_proyectos_filtro_proyecto]).nombre) if origen=="cofinanciables"
  end

  # --
  # METODOS DE GESTION DE Gasto: Listados de gasto, modificar_crear, y eliminar
  # ++

  # en proyectos: lista en función de la etapa y del proyecto
  def listado
    @etapa = Etapa.find( session[:gasto_proyectos_filtro_etapa] ) unless session[:gasto_proyectos_filtro_etapa] == "todas"
    condiciones = { "gasto_x_proyecto.proyecto_id" => @proyecto.id }
    condiciones["gasto.fecha"] = @etapa.fecha_inicio..@etapa.fecha_fin unless session[:gasto_proyectos_filtro_etapa] == "todas"
    condiciones["gasto.moneda_id"] = session[:gasto_proyectos_filtro_moneda] unless session[:gasto_proyectos_filtro_moneda] == "todas"
    condiciones["gasto.agente_id"] = session[:gasto_proyectos_filtro_agente] unless session[:gasto_proyectos_filtro_agente] == "todos"
    condiciones["gasto.partida_id"] = session[:gasto_proyectos_filtro_partida] unless session[:gasto_proyectos_filtro_partida] == "todas"

    @gastos =  Gasto.paginate( :all, :order => (session[:gasto_proyectos_cadena_orden] + " " + session[:gasto_proyectos_asc_desc]), 
                               :include => [:partida, :gasto_x_proyecto, :agente], :conditions => condiciones,
                               :page => (params[:format]=='xls' ? nil : params[:page]), :per_page => (params[:format_xls_count] || session[:gasto_numero]) )

    # en proyectos: en caso de que la seccion sea proyecto hacemos una ultima transformacion a los elementos del listado para obtener el importe por proyecto.
    # Si el parametro select del find funcionase mejor no haria falta hacer a posteriori esto.
    @gastos.each {|p| p.importe_x_proyecto @proyecto }
    elementos_filtrado

    @formato_xls = @gastos.total_entries
    respond_to do |format|
      format.html
      format.xls do
        @tipo = "gasto"
        @objetos = @gastos
        @subobjetos = [ "pago" ]
        render 'comunes_xls/listado', :layout => false
      end
    end
  end

  def datos_formulario
    # Ponemos la fecha inicio y la fecha fin para filtrar en el selector de fechas.
    if session[:gasto_proyectos_filtro_etapa] == "todas" 
      @fecha_inicio = @proyecto.etapa.minimum("fecha_inicio")
      @fecha_fin = @proyecto.etapa.maximum("fecha_fin")
    else
      etapa = Etapa.find( session[:gasto_proyectos_filtro_etapa] )
      @fecha_inicio, @fecha_fin = etapa.fecha_inicio, etapa.fecha_fin
    end
    @partidas = Partida.find(:all).collect {|p| [p.codigo_nombre(@proyecto.id), p.id]}
    @libros = @proyecto.libro.select{|l| session[:usuario_identificado].reload.libro.include? l}.collect {|a|[a.nombre, a.id]}
    @implementadores = @proyecto.implementador.collect {|a|[a.nombre, a.id]}
    @monedas = @proyecto.libro.collect{|l| [l.moneda.abreviatura,l.moneda_id]}.uniq
  end

  def nuevo_copiar_datos
    gasto_copiar, @gasto = Gasto.find(params[:id]), Gasto.new
    Gasto.datos_basicos_igualar(@gasto, gasto_copiar)
    @objeto = @gasto 
    @actividades = gasto_copiar.actividad @proyecto
    @financiadores = gasto_copiar.financiador @proyecto
    datos_formulario
    render :partial => "formulario"
  end

  # en proyectos: prepara el formulario de edición o creación
  def editar_nuevo
    @gasto = @objeto = params[:id] ?  Gasto.find(params[:id]) : Gasto.new
    @gasto_x_proyecto = @gasto.gasto_x_proyecto.find_by_proyecto_id(@proyecto.id)
    @actividades = @gasto.actividad @proyecto
    @financiadores = @gasto.financiador @proyecto
    datos_formulario
    render :partial => "formulario"
  end

  # en proyectos: modifica o crea comprobando que las fechas de la etapa coinciden
  def modificar_crear
    session[:gasto_proyectos_ultimo] = params
    if session[:gasto_proyectos_filtro_etapa] == "todas" 
      fecha_inicio = @proyecto.etapa.minimum("fecha_inicio")
      fecha_fin = @proyecto.etapa.maximum("fecha_fin")
    else
      etapa = Etapa.find( session[:gasto_proyectos_filtro_etapa] )
      fecha_inicio, fecha_fin = etapa.fecha_inicio, etapa.fecha_fin
    end
    #params[:gasto][:fecha] =  params[:gasto][:fecha]
    gasto = params[:id] ?  Gasto.find(params[:id]) : Gasto.new(:proyecto_origen_id => @proyecto.id)
    gasto.attributes = params[:gasto]
    if gasto.comprobar_fecha_etapa fecha_inicio, fecha_fin
      libro = Libro.find_by_id(params[:pago][:libro_id]) if gasto.pago.count <= 1 && params[:pago] && params[:pago][:total] && params[:pago][:libro_id]
      params[:gasto][:moneda_id] = libro.moneda_id if libro
      params[:gasto][:agente_id] = libro.agente_id if libro
      gasto.update_attributes params[:gasto]
      if gasto.errors.empty?
        gxp = gasto.gasto_x_proyecto.find_by_proyecto_id(@proyecto.id) || GastoXProyecto.new(:gasto_id => gasto.id, :proyecto_id => @proyecto.id)
        params[:gasto_x_proyecto][:importe_convertido] = params[:gasto][:importe_convertido]
        gxp.update_attributes params[:gasto_x_proyecto]
        gxp.save
        # Genera un pago unico con fecha del gasto si se ha seleccionado y el gasto es nuevo
        if ( gasto.pago.count <= 1 && params[:pago] && params[:pago][:total] == "1" )
          pago = gasto.pago.first || Pago.new
          pago.importe = gasto.importe
          pago.fecha = gasto.fecha
          pago.observaciones = params[:gasto][:concepto]
          pago.libro_id = params[:pago][:libro_id]
          pago.gasto = gasto
          pago.save
        end
      # Si se manda sin detalle tenemos que especificar e igualar los importes al del presupuesto
        if params["actividades_financiadores"]["detallado"] == "no"
          params["actividades"]["0"]["importe_convertido"], params["financiadores"]["0"]["importe_convertido"] = gasto.importe, gasto.importe 
        end
        gasto.actualizar_gasto_x_actividad(params[:actividades], @proyecto) if params["dividir_actividades"]["todas"] == "0"
        gasto.dividir_por_actividades(@proyecto.actividad, @proyecto) if params["dividir_actividades"]["todas"] == "1"
        gasto.actualizar_gasto_x_agente params[:financiadores], @proyecto
      end
    end
    flash[:objeto_modificado] = {:errors => gasto.errors , :id => gasto.id } if gasto.id
    flash[:objeto_modificado] = {:errors => gasto.errors, :recuperar => true } unless gasto.id
    redirect_to :action => "listado"
  end

  def actividades_financiadores
    @gasto = @objeto = params[:id] ?  Gasto.find(params[:id]) : Gasto.new
    @actividades = @gasto.actividad @proyecto
    @financiadores = @gasto.financiador @proyecto
    render :update do |page|
      page.replace "actividades_financiadores", :partial => "comunes/actividades_financiadores"
      page.call("Modalbox.resizeToContent")
    end 
  end


  # en proyectos: añade un financiador al formulario
  def anadir_financiador
    render :template => "comunes/anadir_financiador"
  end


  # en proyectos: añade una actividad al formulario
  def anadir_actividad
    render :template => "comunes/anadir_actividad"
  end

  # en proyectos: elimina
  def eliminar
    Gasto.find(params[:id]).destroy
    flash[:mensaje] = _("Se ha eliminado el gasto.")
    redirect_to :action => 'ordenado'
  end


  # --
  # METODOS DE GESTION DE GASTOS COFINANCIADOS.
  # --  

	# en proyectos: muestra el listado de todos los gastos disponibles para vincular
  def cofinanciables
    @etapa = Etapa.find( session[:gasto_proyectos_filtro_etapa] ) unless session[:gasto_proyectos_filtro_etapa] == "todas"
    condiciones = {"proyecto_origen_id" => @proyecto.proyecto_cofinanciador}
    condiciones["gasto.fecha"] = @etapa.fecha_inicio..@etapa.fecha_fin unless session[:gasto_proyectos_filtro_etapa] == "todas"
    condiciones["gasto.partida_id"] = session[:gasto_proyectos_filtro_partida] unless session[:gasto_proyectos_filtro_partida] == "todas"
    condiciones["gasto.proyecto_origen_id"] = session[:gasto_proyectos_filtro_proyecto] unless session[:gasto_proyectos_filtro_proyecto] == "todos"
    # Si el filtro de moneda es todas, escogemos solo aquellas aceptadas por el proyecto
    condiciones["gasto.moneda_id"] = @proyecto.moneda if session[:gasto_proyectos_filtro_moneda] == "todas"
    condiciones["gasto.moneda_id"] = session[:gasto_proyectos_filtro_moneda] unless session[:gasto_proyectos_filtro_moneda] == "todas"
    # Si el filtro de agente es todos, escogemos solo aquellos aceptados por el proyecto
    condiciones["gasto.agente_id"] = @proyecto.implementador if session[:gasto_proyectos_filtro_agente] == "todos"
    condiciones["gasto.agente_id"] = session[:gasto_proyectos_filtro_agente] unless session[:gasto_proyectos_filtro_agente] == "todos"

    @gastos =  Gasto.paginate(	:all, :order => (session[:gasto_proyectos_cadena_orden] + " " + session[:gasto_proyectos_asc_desc]),
				:include => [:partida, :agente, :gasto_x_proyecto], :conditions => condiciones,
				:page => (params[:format]=='xls' ? nil : params[:page]), :per_page => (params[:format_xls_count] || session[:gasto_numero]) )

    elementos_filtrado

    @formato_xls = @gastos.total_entries
    respond_to do |format|
      format.html
      format.xls do
        @tipo = "gasto"
        @objetos = @gastos
        @subobjetos = [ "pago" ]
        render 'comunes_xls/listado', :layout => false
      end
    end
 
  end

	# en proyectos: desasigna un gasto del proyecto
  def descofinanciar
    gasto = Gasto.find_by_id params[:id]
    gxp = gasto.gasto_x_proyecto.find_by_proyecto_id(@proyecto.id) if gasto
    if gxp
      gasto.gasto_x_actividad.find_all_by_proyecto_id(@proyecto.id).each { |gxa| gxa.destroy }
      gasto.gasto_x_agente.find_all_by_proyecto_id(@proyecto.id).each { |gxa| gxa.destroy }
      gxp.destroy 
      flash[:error] = gxp
    end
    redirect_to :action => params[:update]
  end

  # --
  # METODOS DE GESTION DE FACTURAS: orden de facturas
  # --

  # en proyectos: muestra el listado de facturas
  def facturas
    session[:gasto_proyectos_filtro_etapa] = @proyecto.etapa.sort{ |a,b| a.fecha_inicio <=> b.fecha_inicio }.first.id
    redirect_to :action => :listado_facturas
  end


  def listado_facturas
    @etapa = Etapa.find( session[:gasto_proyectos_filtro_etapa] ) unless session[:gasto_proyectos_filtro_etapa] == "todas"
    condiciones = { "gasto_x_proyecto.proyecto_id" => @proyecto.id }
    condiciones["gasto.fecha"] = @etapa.fecha_inicio..@etapa.fecha_fin unless session[:gasto_proyectos_filtro_etapa] == "todas"
    condiciones["gasto.moneda_id"] = session[:gasto_proyectos_filtro_moneda] unless session[:gasto_proyectos_filtro_moneda] == "todas"
    condiciones["gasto.agente_id"] = session[:gasto_proyectos_filtro_agente] unless session[:gasto_proyectos_filtro_agente] == "todos"
    condiciones["gasto.partida_id"] = session[:gasto_proyectos_filtro_partida] unless session[:gasto_proyectos_filtro_partida] == "todas"
    @facturas =  GastoXProyecto.paginate( :all, :order => ("orden_factura"), :include => [:gasto],
                                         :conditions => condiciones,
                                         :page => params[:page], :per_page => session[:gasto_numero] )

    elementos_filtrado
  end

  # en proyectos: ordena las facturas
  def orden_facturas
    if params[:selector]
      condiciones = { "gasto_x_proyecto.proyecto_id" => @proyecto.id }
      condiciones["gasto.fecha"] = @etapa.fecha_inicio..@etapa.fecha_fin unless session[:gasto_proyectos_filtro_etapa] == "todas"
      condiciones["gasto.moneda_id"] = session[:gasto_proyectos_filtro_moneda] unless session[:gasto_proyectos_filtro_moneda] == "todas"
      condiciones["gasto.agente_id"] = session[:gasto_proyectos_filtro_agente] unless session[:gasto_proyectos_filtro_agente] == "todos"
      condiciones["gasto.partida_id"] = session[:gasto_proyectos_filtro_partida] unless session[:gasto_proyectos_filtro_partida] == "todas"
      orden = (params[:selector][:tipo] == "partida" && params[:selector][:tipo] == "subpartida")? "partida.id,fecha" : "fecha"
      @gastos =  Gasto.find( :all, :order => orden, :include => [:partida, :gasto_x_proyecto], :conditions => condiciones )
      contador = {} 
      prefijo = @proyecto.nombre.to_s
      contador[prefijo] = 1
      @gastos.each do |gasto|
        gasto_x_proyecto = GastoXProyecto.find :first, :conditions => { 'proyecto_id' => @proyecto.id, 'gasto_id' => gasto.id }
        # En el caso de que el orden sea por partidas tenemos que cambiar el codigo de numeracion con cada una
        if (params[:selector][:tipo] == "partida" || params[:selector][:tipo] == "subpartida") 
          partida = gasto.partida.partida_asociada @proyecto if gasto.partida
          subpartida = Subpartida.find(gasto_x_proyecto.subpartida_id) if gasto_x_proyecto.subpartida_id
          if !partida
            prefijo=nil
          #elsif subpartida.nil? && params[:selector][:tipo] == "subpartida"
          #  prefijo=nil
          else
            if (@proyecto.nombre.to_s + "/" + partida.codigo.to_s + ((params[:selector][:tipo] == "subpartida" && !subpartida.nil?) ? "/" + subpartida.numero.to_s : "") != prefijo)
              prefijo = @proyecto.nombre.to_s + "/" + partida.codigo.to_s + ((params[:selector][:tipo] == "subpartida" && !subpartida.nil?) ? "/" + subpartida.numero.to_s : "")
              contador[prefijo] = 1 if !contador[prefijo]
            end
          end
        end
        gasto_x_proyecto.orden_factura = prefijo ? prefijo + "/" + format("%03d",contador[prefijo].to_s) : ""
        contador[prefijo] += 1 if prefijo
        gasto_x_proyecto.save
      end  
    end
    redirect_to :action => :listado_facturas
  end

        # en proyectos: genera la etiqueta para pegar a la factura
  def generar_etiqueta
    factura = GastoXProyecto.find_by_id(params[:id])
    if factura
      partida_asociada = factura.gasto.partida.partida_asociada(@proyecto) if factura.gasto.partida && factura.gasto.partida.partida_asociada(@proyecto)
      imagen = Magick::Image.new(500,280)
      imagen.format = 'PNG'

      text = Magick::Draw.new
      text.font_family = 'helvetica'
      text.pointsize = 24 
      text.gravity = Magick::NorthGravity
      posicion = 30
      word_wrap(factura.orden_factura, 30).split("\n").each do |row|
        text.annotate(imagen, 0, 0, 0, posicion += 30, row)
      end

      text.pointsize = 20 
      text.align = Magick::LeftAlign
      texto  = "Fecha: " + factura.gasto.fecha.to_s + "\n"
      texto += "Factura: " + (factura.gasto.numero_factura || "") + "\n"
      texto += "Partida: " + ( partida_asociada ? partida_asociada.codigo + " (" + partida_asociada.nombre.upcase + ")" : "N/A" ) + "\n"
      texto += "Subpartida: " + Subpartida.find(factura.subpartida_id).nombre.upcase if factura.subpartida_id
  
      posicion = 140 
      word_wrap(texto, 30).split("\n").each do |row|
        text.annotate(imagen, 0, 0, 20, posicion += 20, row)
      end   

      send_data imagen.to_blob, :filename => 'Etiqueta_' + factura.orden_factura + '.png', :disposition => 'inline', :type => 'application/png'
    end
  end

  # --
  # METODOS AJAX DEL FORMULARIO: Moneda y subpartida.
  # ++

  # en proyectos: devuelve la moneda del libro seleccionado para el fomulario
  def moneda_libro
    @moneda = params[:id] != "" ? Libro.find(params[:id]).moneda : nil
    render :inline => <<-FIN
      <%= _("Moneda") %> <br> <%=  @moneda.abreviatura if @moneda %>
      <%= hidden_field('gasto', 'moneda_id', :value => @moneda.id) if @moneda%>
    FIN
  end

  # en proyectos: devuelve las subpartidas asociadas a la partida seleccionada para el fomulario
  def subpartida
    render :partial => "subpartida", :locals => { :partida_id => params[:id]}
  end

  # --
  # METODOS DE MODIFICACION PARCIAL DE LA INFORMACION DE GASTOS DE VARIOS PROYECTOS
  # ++

  def editar_parcial
    @gasto = @objeto = params[:id] ?  Gasto.find(params[:id]) : Gasto.new
    @gasto_x_proyecto = @gasto.gasto_x_proyecto.find_by_proyecto_id(@proyecto.id) || @gasto.gasto_x_proyecto.new(:proyecto_id => @proyecto.id, :importe => @gasto.importe)
    @actividades = @gasto.actividad @proyecto
    @financiadores = @gasto.financiador @proyecto
    datos_formulario
    render :partial => "formulario_parcial"
  end

        # en proyectos: modifica un gasto cofinanciado
  def modificar_parcial
    @etapa = Etapa.find_by_id( session[:gasto_proyectos_filtro_etapa] )
    gasto = Gasto.find_by_id params[:id]
    if gasto
      gxp = gasto.gasto_x_proyecto.find_by_proyecto_id(@proyecto.id) || (gasto.proyecto_origen_id ? gasto.gasto_x_proyecto.new(:proyecto_id => @proyecto.id,:importe => gasto.importe) : nil)
      gxp.update_attributes params[:gasto_x_proyecto] if gxp
      gxp.save if gxp
      if gxp && gxp.errors.empty?
        gasto.actualizar_gasto_x_actividad params[:actividades], @proyecto
        gasto.actualizar_gasto_x_agente params[:financiadores], @proyecto
        flash[:error] = gasto
      else
        flash[:error] = gxp if gxp
        flash[:error] = "El gasto no esta asociado a este proyecto" unless gxp
      end
    end
    redirect_to :action => params[:update] 
  end

  #-- 
  # METODOS DE RECUPERAR EL ULTIMO GASTO EDITADO
  # ++

  def editar_ultimo
    @gasto = @objeto = Gasto.new(session[:gasto_proyectos_ultimo]["gasto"])
    @actividades = [GastoXActividad.new(session[:gasto_proyectos_ultimo]["actividades"]["0"])]
    @financiadores = [GastoXAgente.new(session[:gasto_proyectos_ultimo]["financiadores"]["0"])]
    datos_formulario
    render :partial => "formulario"
  end

  private
    def word_wrap(text, columns = 80)
      text.split("\n").collect do |line|
        line.length > columns ? line.gsub(/(.{1,#{columns}})([\s\/]+|$)/, "\\1\n").strip : line
      end * "\n"
    end
end

