#--
#
#################################################################################
# Copyright 2011 CENATIC y IEPALA
#
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas por la Comisión Europea– 
# versiones posteriores de la EUPL (la «Licencia»);
# Solo podrá usarse esta obra si se respeta la Licencia.
# Puede obtenerse una copia de la Licencia en:
#
# http://www.osor.eu/eupl/european-union-public-licence-eupl-v.1.1
#
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye «TAL CUAL»,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones que establece la Licencia.
#################################################################################
#
#++
# Controlador encargado de la gestion de la entidad partida Financiacion.
# Estas partidas son las partidas de las financiaciones que se mapean con las partidas generales del sistema
# Este controlador es utilizado desde las secciones:
# * Sección financiaciones: se utiliza para sacar los informes para los financiacores

class PartidaFinanciacionController < ApplicationController

  # en financiaciones: se redirecciona por defecto a listado
  def index
    redirect_to :action => 'listado'
  end

  # --
  # METODOS DE GESTION DE Partidas
  # ++

  # en financiaciones: lista las partidas de la finaciación cargada en la sessión
  def listado
    @partidas_financiacion = @proyecto.partida_financiacion.find(:all,:order => 'codigo')
    Partida.all.each do |partida|
      flash[:error] = _("Alguna partida de sistema no está aún mapeada") if partida.partida_asociada(@proyecto).nil?
    end

    @formato_xls = true
    respond_to do |format|
      format.html
      format.xls do
        @tipo = "partida_financiacion"
        @objetos = @partidas_financiacion
        @subobjetos = [ "partida" ]
        render 'comunes_xls/listado', :layout => false
      end
    end
  end

  # en financiaciones: prepara el formulario de edición o creación
  def editar_nuevo
    @partida_financiacion = params[:id] ?  PartidaFinanciacion.find(params[:id]) : nil
    @partidas_financiacion_padre = PartidaFinanciacion.find_all_by_padre( true ).collect{ |p| [p.nombre, p.id]}
    render :partial => "formulario"
  end

  # en financiaciones: modifica o crea
  def modificar_crear
    @partida_financiacion = params[:id] ?  PartidaFinanciacion.find(params[:id]) : PartidaFinanciacion.new
    params[:partida_financiacion][:proyecto_id] = @proyecto.id
    @partida_financiacion.update_attributes params[:partida_financiacion]
    flash[:error] = @partida_financiacion
    redirect_to :action => "listado"
  end

  # en financiaciones:  elimina
  def eliminar
    PartidaFinanciacion.find(params[:id]).destroy
    flash[:mensaje] = _("Se ha eliminado la partida de la financiación.")
    redirect_to :action => 'listado'
  end

  #-- 
  # METODOS DE PARTIDAS ASOCIADAS CON PARTIDA_FINANCIACION -- Metodos AJAX 
  # ++

  # en financiaciones: lista las partidas del sistema asociadas
  def partidas_asociadas
     @partidas_asociadas = PartidaFinanciacion.find( params[:partida_financiacion_id] ).partida
    render :update do |page|
      page.replace_html params[:update], :partial => "partidas_asociadas", :locals => { :update => params[:update], :partida_financiacion_id => params[:partida_financiacion_id] }
    end
  end

  # en financiaciones: prepara el formulario para asociar partida del sistema
  def asociar_partida
    @partidas_financiacion = PartidaFinanciacion.find( params[:partida_financiacion_id] )
    @partidas_pendientes = Partida.partidas_pendientes_de_financiacion(@proyecto.id).collect {|e| [e.codigo_nombre ,e.id]}
    render :partial => "asociar_partida"
  end


  # en financiaciones: asocia las partidas del sistema
  def crear_asociacion_partida
    if ! params[:partida][:id].empty?
      @partidas_asociadas =  PartidaFinanciacion.find( params[:partida_financiacion_id] ).partida
      @partida_financiacion = @objeto = PartidaFinanciacion.find( params[:partida_financiacion_id] )
      @partida_financiacion.partida << Partida.find( params[:partida][:id] )
      @partial, @locales = "partidas_asociadas",  { :update => params[:update], :partida_financiacion_id => params[:partida_financiacion_id] }
      render :template => "partida_financiacion/mensaje_formulario_cambio_sublistado", :locals => { :update => params[:update] }
    else
      render :update do |page|
        page.alert('Ninguna partida fue asociada')
        page.call("Modalbox.hide")
      end
    end
  end

  # en financiaciones: desasocia la partida
  def desasociar_partida
    partida_financiacion = PartidaFinanciacion.find( params[:partida_financiacion_id] )
    partida_financiacion.partida.delete( @partida = Partida.find( params[:partida_id] ) )
    @partidas_asociadas = PartidaFinanciacion.find( params[:partida_financiacion_id] ).partida
    render :update do |page|
      page.replace_html params[:update], :partial => "partidas_asociadas", :locals => { :update => params[:update], :partida_financiacion_id => params[:partida_financiacion_id] }
      page.visual_effect :highlight, params[:update] , :duration => 6
      page.replace_html 'MB_content', :inline => '<%= mensaje_error(@partida, :eliminar => true) %><br>'
      page.call("Modalbox.resizeToContent")
    end
  end

end
