#--
#
#################################################################################
# Copyright 2011 CENATIC y IEPALA
#
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas por la Comisión Europea– 
# versiones posteriores de la EUPL (la «Licencia»);
# Solo podrá usarse esta obra si se respeta la Licencia.
# Puede obtenerse una copia de la Licencia en:
#
# http://www.osor.eu/eupl/european-union-public-licence-eupl-v.1.1
#
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye «TAL CUAL»,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones que establece la Licencia.
#################################################################################
#
#++
# gasto

class Gasto < ActiveRecord::Base

  untranslate_all

  
  belongs_to :moneda
  belongs_to :agente
  belongs_to :partida
  belongs_to :marcado
  belongs_to :subpartida_agente, :class_name => "Subpartida", :foreign_key => "subpartida_agente_id"

  has_many :gasto_x_proyecto, :dependent => :destroy
  has_many :gasto_x_agente, :dependent => :destroy
  has_many :gasto_x_actividad, :dependent => :destroy
  has_many :gasto_x_transferencia, :dependent => :destroy
  has_many :tasa_cambio_gasto, :dependent => :destroy
  has_many :pago, :dependent => :destroy
  
  has_and_belongs_to_many :transferencia, :join_table =>  "gasto_x_transferencia"

  #named_scope :vinculable_al_proyecto, lambda { |p| { :conditions => ["`proyecto_origen_id` IN (?)",p.proyecto_cofinanciador]} }

  validates_presence_of :importe, :message => _("Importe") + _(" no pueden estar vacío.")
  validates_presence_of :moneda_id, :message => _("Moneda") + _(" no pueden estar vacío.")
  validates_presence_of :partida_id, :message => _("Partida") + _(" no pueden estar vacío.")
  validates_presence_of :agente_id, :message => _("Agente Implementador") + _(" no pueden estar vacío.")
  validates_presence_of :fecha,  :message => _("Fecha") + _(" no pueden estar vacío.")  
  validates_associated :agente, :message =>  _("El agente implementador asociado no es correcto.")

  validate :comprueba_libro_moneda

  # Comprueba que el libro de cada pago pertenezca al agente implementador
  # y que el libro tiene la misma moneda que el gasto
  def comprueba_libro_moneda
    # Con mirar el primero vale
    p=pago.first
    if p && p.libro
      errors.add("Pago", _("La cuenta de algún pago no pertenece al agente. Elimine los pagos existentes antes de cambiar el agente.")) if p.libro.agente != agente
      errors.add("Pago", _("Existe algún pago con moneda distinta al gasto. Elimine los pagos existentes antes de cambiar la moneda.")) if p.libro.moneda != moneda
    end
  end

  def subpartida_proyecto proyecto
    gasto_x_proyecto.find_by_proyecto_id(proyecto).subpartida
  end
  def subpartida_proyecto_nombre proyecto
    subpartida = subpartida_proyecto proyecto
    subpartida ? subpartida.nombre : nil
  end


# De momento comentamos los temas relacionados con informes hasta que tengamos claro el sentido que va a tomar.
#  before_save :fecha_y_fecha_informe
#  def fecha_y_fecha_informe
#    self.fecha_informe = self.fecha if self.fecha_informe.nil?
#  end

  def subpartidas_nombres
    subpartida.nombre if subpartida
  end 

   def subpartida_nombre=(nombre)
     self.subpartida=Subpartida.find_or_create_by_nombre(nombre) unless nombre.blank?
   end 

  # Comprueba que la fecha esta dentro de la etapa selecionada. 
  def comprobar_fecha_etapa fecha_inicio, fecha_fin
    if self.fecha and self.fecha > fecha_inicio and self.fecha < fecha_fin
      return true
    else
      errors.add(_("Fecha"), _("La fecha debe estar dentro de la etapa")) ; return false
    end
  end

  # Devuelve el nombre del proyecto origen del gasto
  def proyecto_origen
    Proyecto.find_by_id(proyecto_origen_id).nombre if proyecto_origen_id
  end

  # Devuelve los financiadores del gasto para un determinado proyecto   
  def financiador proyecto
     gasto_x_agente.find :all, :conditions => {"proyecto_id" => ( proyecto.class != Proyecto ? proyecto.to_i : proyecto.id )}
  end

  # Devuelve los actividades del gasto para un determinado proyecto
  def actividad proyecto
     gasto_x_actividad.find :all, :conditions => {"proyecto_id" => ( proyecto.class != Proyecto ? proyecto.to_i : proyecto.id )}
  end

  # Devuelve un array con los importes por actividad
  def importes_por_actividades proyecto=nil
    salida = Array.new
    condiciones = Hash.new
    condiciones["proyecto_id"] = (proyecto.class != Proyecto ? proyecto.to_i : proyecto.id) if proyecto
    gxas = gasto_x_actividad.find(:all, :conditions => condiciones)
    gxas.each {|gxa| salida.push( gxa.actividad.codigo + ": " + gxa.importe.to_s ) }
    return salida
  end

  # Devuelve un array con los importes por financiador
  def importes_por_financiadores proyecto=nil
    salida = Array.new
    condiciones = Hash.new
    condiciones["proyecto_id"] = (proyecto.class != Proyecto ? proyecto.to_i : proyecto.id) if proyecto
    gxfs = gasto_x_agente.find(:all, :conditions => condiciones)
    gxfs.each {|gxf| salida.push( gxf.agente.nombre + ": " + gxf.importe.to_s ) }
    return salida
  end

  # Devuelve un array con los importes por proyectos
  def importes_por_proyectos
    gasto_x_proyecto.collect { |gxp| gxp.proyecto.nombre + ": " + (gxp.importe).to_s }
  end

  def todas_actividades? proyecto
    return gasto_x_actividad.find_all_by_proyecto_id(proyecto.id).size == proyecto.actividad.size if proyecto
  end

   # Moneda del gasto
   def moneda
     return self.moneda_id ? Moneda.find(self.moneda_id) : nil
   end

   #Metodo especifico creado para el ruport y la exportacion
   def moneda_abreviatura
     return Moneda.find(self.moneda_id).abreviatura if self.moneda_id
   end

   #Metodo especifico creado para el ruport y la exportacion
   def partida_codigo_nombre_aborrar
     return partida.codigo_nombre if self.partida_id
   end

   # importe para el proyecto asociado al gasto
   def importe_x_proyecto proyecto
     gxp = gasto_x_proyecto.detect {|gxp| gxp.proyecto_id == proyecto.id}
     if gxp
       self.importe = gxp.importe
     end
   end
  
   # Actualiza el listado de agentes con los que se ha asociado el gasto para un determinado proyecto.
   def actualizar_gasto_x_agente listado, proyecto
     financiador(proyecto).each {|f| f.destroy}
     listado.each do |key, value|
       gxa = gasto_x_agente.create(value)
       errors.add("", gxa.errors.inject('') {|total, e| total + e[1]  }) unless gxa.errors.empty?
     end
     total = self.gasto_x_proyecto.find_by_proyecto_id(proyecto.id).importe
     if  listado.inject(0) {|suma, value| suma + value.pop[:importe_convertido].to_f} != total
       errors.add( _('Importes'), _("Los importes de los financiadores no suman el importe total del gasto."))        
     end
   end

   # Actualiza el listado de actividades con los que se ha asociado el gasto para un determinado proyecto.
   def actualizar_gasto_x_actividad listado, proyecto
     actividad(proyecto).each {|f| f.destroy}
     listado.each do |key, value|
       gxa = gasto_x_actividad.create(value)
       errors.add( "", gxa.errors.inject('') {|total, e| total + e[1]  } ) unless gxa.errors.empty?
     end        
     total = self.gasto_x_proyecto.find_by_proyecto_id(proyecto.id).importe
     if  listado.inject(0) {|suma, value| suma + value.pop[:importe_convertido].to_f} != total
       errors.add( _('Importes'), _("Los importes de las actividades no suman el importe total del gasto.") )        
     end
   end

   # Actualiza los proyectos a los que se ha asociado el gasto.
   def actualizar_gasto_x_proyecto listado
    #if  listado.inject(0) {|suma, value| suma + value.pop[:porcentaje].to_f} == 1
      gasto_x_proyecto.clear
      listado.each do |key, value|
        gxp = gasto_x_proyecto.create(value)
        errors.add( "", gxp.errors.inject('') {|total, e| total + e[1]  } ) unless gxp.errors.empty?
      end        
    #else
    #  errors.add( _('Porcentajes'), _("El porcentaje de los proyectos asociados al gasto debe sumar 100. No se guardan los datos.") )        
    #end
  end

  def dividir_por_actividades actividades, proyecto
     detalle = Hash.new
     importe_x_actividad, resto_importe = importe.divmod(actividades.size)
     #numero_unidades_x_actividad, resto_numero_unidades = numero_unidades.divmod(actividades.size)
     for actividad in actividades
        if actividad == actividades.last
          importe_x_actividad += resto_importe
          #numero_unidades_x_actividad += resto_numero_unidades
        end
        detalle[actividad.id] = { :importe_convertido => importe_x_actividad,  :actividad_id => actividad.id, :proyecto_id => proyecto.id}  
     end
     actualizar_gasto_x_actividad detalle, proyecto
  end

  # Devuelve la tasa de cambio para un determinado proyeto.
  def tasa_cambio_proyecto proyecto
    return tasa_cambio_gasto.detect {|tcg| tcg.proyecto_id == proyecto} 
  end

  def tasa_cambio_agente
    return tasa_cambio_gasto.detect {|tcg| tcg.agente_id == self.agente_id} 
  end

  # Devuelve la identificacion del gasto para un determinado proyeto y una determinada financiación.
  def identificacion_gasto_aborrar proyecto, financiacion
    return gasto_identificacion.detect {|ig| ig.proyecto_id == proyecto and ig.financiacion_id == financiacion}
  end

  # Comprueba si el gasto esta asociado correctamente a proyectos.
  def comprobar_proyectos
    return gasto_x_proyecto.inject(0) {|suma, gxp| suma + gxp.importe} == self.importe 
  end

  # Comprueba si el gasto esta asociado correctamente a proyectos.
  def comprobar_financiadores proyecto
    return  financiador(proyecto).inject(0) {|suma, f| suma + f.importe} == self.importe 
  end

  # Comprueba si el gasto esta asociado correctamente a proyectos.
  def comprobar_actividades proyecto
    return  actividad(proyecto).inject(0) {|suma, f| suma + f.importe} == self.importe 
  end

  #--
  # FINDERS complicados... Mas alla de ActiveRecordBase
  # ++

  # Busqueda creada manualmente para los detalles de busquedas especificas que no cubre el find habitual de ActiveRecord.
  def self.busqueda campos, condiciones, asociacion, agrupado, ordenado
    consulta = " SELECT " + campos + " FROM gasto  "
    consulta += asociaciones asociacion
    consulta += " WHERE " + merge_conditions(condiciones) if condiciones
    consulta += " GROUP BY " + agrupado if agrupado
    consulta += " ORDER BY " + ordenado if ordenado
    return find_by_sql consulta 
  end
  
  # Cutrada para sustituir el include para el metodo busqueda.
  def self.asociaciones asociacion
    asociacion.inject(' ') do |suma, elemento|
      case elemento
        #when :libro
        #suma + ' LEFT OUTER JOIN libro on gasto.libro_id = libro.id'
        when :gasto_x_proyecto
        suma + ' LEFT OUTER JOIN gasto_x_proyecto on gasto_x_proyecto.gasto_id = gasto.id'
        when :tasa_cambio_gasto
        suma + ' LEFT OUTER JOIN tasa_cambio_gasto on tasa_cambio_gasto.gasto_id = gasto.id'
        when :gasto_x_agente
        suma + ' LEFT OUTER JOIN gasto_x_agente on gasto_x_agente.gasto_id = gasto.id'
        when :gasto_x_actividad
        suma + ' LEFT OUTER JOIN gasto_x_actividad on gasto_x_actividad.gasto_id = gasto.id'
        when :partida_x_partida_financiacion
        suma + ' LEFT OUTER JOIN partida_x_partida_financiacion on partida_x_partida_financiacion.partida_id = gasto.partida_id'
        when :partida_financiacion
        suma + ' LEFT OUTER JOIN partida_financiacion on partida_financiacion.id = partida_x_partida_financiacion.partida_financiacion_id'
        #when :gasto_identificacion
        #suma + ' LEFT OUTER JOIN gasto_identificacion on gasto_identificacion.gasto_id = gasto.id'
      end
    end
  end

  # Metodo que iguala algunos valores basicos para procesos de insercion de datos.
  def self.datos_basicos_igualar gasto, gasto_anterior
    gasto.partida = gasto_anterior.partida
    gasto.fecha = gasto_anterior.fecha
    #gasto.libro_id = gasto_anterior.libro_id
    gasto.agente_id = gasto_anterior.agente_id
    gasto.moneda_id = gasto_anterior.moneda_id

  end



end
