#--
#
#################################################################################
# Copyright 2011 CENATIC y IEPALA
#
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas por la Comisión Europea– 
# versiones posteriores de la EUPL (la «Licencia»);
# Solo podrá usarse esta obra si se respeta la Licencia.
# Puede obtenerse una copia de la Licencia en:
#
# http://www.osor.eu/eupl/european-union-public-licence-eupl-v.1.1
#
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye «TAL CUAL»,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones que establece la Licencia.
#################################################################################
#
#++

# Clase del modelo que gestiona la entidad PresupuestoXActividad que recoge la relacion entre presupuesto y actividad con porcentaje
class PresupuestoXActividad < ActiveRecord::Base
  untranslate_all
  belongs_to :presupuesto
  belongs_to :actividad

  validates_presence_of :actividad, :message => _("Actividad") + _(" no es válida.")
  validates_uniqueness_of :actividad_id, :scope => [:presupuesto_id], :message => _("Actividad") + _(" repetida.")
  validates_numericality_of :importe, :greater_than => 0, :message =>_("Actividad") + _(" con importe 0 no se guarda.")
  validates_numericality_of :numero_unidades, :greater_than_or_equal_to => 0, :message =>_("Numero de unidades") + _(" debe ser mayor que cero.")

  validate :comprobar_etapa

  # Comprueba que la etapa asignada a la linea de presupuesto esta permitida para la actividad seleccionada
  def comprobar_etapa
    errors.add("Etapa", _("La actividad ") + "\"" + actividad.codigo + "\"" + _(" no está asignada a la etapa ") + "\"" + presupuesto.etapa.nombre + "\"" ) if actividad && (actividad.etapa.nil? || actividad.etapa.find_by_id(presupuesto.etapa_id).nil? )
  end

end
