#--
#
#################################################################################
# Copyright 2011 CENATIC y IEPALA
#
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas por la Comisión Europea– 
# versiones posteriores de la EUPL (la «Licencia»);
# Solo podrá usarse esta obra si se respeta la Licencia.
# Puede obtenerse una copia de la Licencia en:
#
# http://www.osor.eu/eupl/european-union-public-licence-eupl-v.1.1
#
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye «TAL CUAL»,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones que establece la Licencia.
#################################################################################
#
#++

# Clases del modelo que gestiona la entidad Transferencia.

class Transferencia < ActiveRecord::Base

  untranslate_all

  belongs_to :moneda
  belongs_to :libro
  belongs_to :etapa
  belongs_to :agente
  belongs_to :proyecto
  has_and_belongs_to_many :gasto, :join_table =>  "gasto_x_transferencia"
  has_many :gasto_x_transferencia, :dependent => :destroy

  validates_presence_of :importe, :message => _("Importe") + _(" no puede estar vacío.")
  validates_presence_of :moneda_id, :message => _("Moneda") + _(" no puede estar vacío.")
  validates_presence_of :libro_id, :message => _("Cuenta") + _(" no puede estar vacío.")
  validates_presence_of :entrante_saliente, :message => _("Tipo") + _(" no puede estar vacío.")
  validates_presence_of :tipo, :message => _("Tipologia de movimiento no definida")
  validates_presence_of :fecha,  :message => _("Fecha") + _(" no puede estar vacío.")

  # Comprueba que la fecha está dentro de la etapa seleccionada
  def comprobar_fecha_etapa etapa
    if self.fecha > etapa.fecha_inicio and self.fecha < etapa.fecha_fin
      return true
    else
      errors.add(_("Fecha"), _("La fecha debe estar dentro de la etapa")) ; return false
    end
  end
  # Moneda de la transferencia
  def moneda
    return self.moneda_id ? Moneda.find(self.moneda_id) : nil
  end
  # Proyecto al que se ha asignado la transferencia
  def proyecto
    return Proyecto.find(self.proyecto_id) if self.proyecto_id  
  end

  # Devuelve el tipo completo de la transferencia
  def tipo_completo
    return self.tipo.capitalize + " " + self.entrante_saliente unless self.tipo == "intereses" || self.tipo == "subvencion"
    return self.tipo.capitalize if self.tipo == "intereses" || self.tipo == "subvencion"
  end

  # Libro receptor o emisor (segun el tipo de la transferencia)
  def  libro_receptor_emisor
     return Libro.find(self.libro_receptor_emisor_id) if self.libro_receptor_emisor_id
  end

end
